@@ -11,9 +11,9 @@ import (
 	"strings"
 	"sync"
 
+	"github.com/jesseduffield/generics/set"
 	"github.com/jesseduffield/lazygit/pkg/commands/models"
 	"github.com/jesseduffield/lazygit/pkg/commands/oscommands"
-	"github.com/jesseduffield/lazygit/pkg/commands/types/enums"
 	"github.com/jesseduffield/lazygit/pkg/common"
 	"github.com/jesseduffield/lazygit/pkg/utils"
 	"github.com/samber/lo"
@@ -31,10 +31,10 @@ type CommitLoader struct {
 	*common.Common
 	cmd oscommands.ICmdObjBuilder
 
-	getRebaseMode func() (enums.RebaseMode, error)
-	readFile      func(filename string) ([]byte, error)
-	walkFiles     func(root string, fn filepath.WalkFunc) error
-	dotGitDir     string
+	getWorkingTreeState func() models.WorkingTreeState
+	readFile            func(filename string) ([]byte, error)
+	walkFiles           func(root string, fn filepath.WalkFunc) error
+	dotGitDir           string
 	*GitCommon
 }
 
@@ -42,16 +42,16 @@ type CommitLoader struct {
 func NewCommitLoader(
 	cmn *common.Common,
 	cmd oscommands.ICmdObjBuilder,
-	getRebaseMode func() (enums.RebaseMode, error),
+	getWorkingTreeState func() models.WorkingTreeState,
 	gitCommon *GitCommon,
 ) *CommitLoader {
 	return &CommitLoader{
-		Common:        cmn,
-		cmd:           cmd,
-		getRebaseMode: getRebaseMode,
-		readFile:      os.ReadFile,
-		walkFiles:     filepath.Walk,
-		GitCommon:     gitCommon,
+		Common:              cmn,
+		cmd:                 cmd,
+		getWorkingTreeState: getWorkingTreeState,
+		readFile:            os.ReadFile,
+		walkFiles:           filepath.Walk,
+		GitCommon:           gitCommon,
 	}
 }
 
@@ -60,27 +60,26 @@ type GetCommitsOptions struct {
 	FilterPath           string
 	FilterAuthor         string
 	IncludeRebaseCommits bool
-	RefName              string // e.g. "HEAD" or "my_branch"
-	RefForPushedStatus   string // the ref to use for determining pushed/unpushed status
+	RefName              string     // e.g. "HEAD" or "my_branch"
+	RefForPushedStatus   models.Ref // the ref to use for determining pushed/unpushed status
 	// determines if we show the whole git graph i.e. pass the '--all' flag
 	All bool
 	// If non-empty, show divergence from this ref (left-right log)
 	RefToShowDivergenceFrom string
 	MainBranches            *MainBranches
+	HashPool                *utils.StringPool
 }
 
 // GetCommits obtains the commits of the current branch
 func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit, error) {
 	commits := []*models.Commit{}
-	var rebasingCommits []*models.Commit
 
 	if opts.IncludeRebaseCommits && opts.FilterPath == "" {
 		var err error
-		rebasingCommits, err = self.MergeRebasingCommits(commits)
+		commits, err = self.MergeRebasingCommits(opts.HashPool, commits)
 		if err != nil {
 			return nil, err
 		}
-		commits = append(commits, rebasingCommits...)
 	}
 
 	wg := sync.WaitGroup{}
@@ -91,30 +90,34 @@ func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit,
 	go utils.Safe(func() {
 		defer wg.Done()
 
-		logErr = self.getLogCmd(opts).RunAndProcessLines(func(line string) (bool, error) {
-			commit := self.extractCommitFromLine(line, opts.RefToShowDivergenceFrom != "")
-			commits = append(commits, commit)
-			return false, nil
+		var realCommits []*models.Commit
+		realCommits, logErr = loadCommits(self.getLogCmd(opts), opts.FilterPath, func(line string) (*models.Commit, bool) {
+			return self.extractCommitFromLine(opts.HashPool, line, opts.RefToShowDivergenceFrom != ""), false
 		})
-	})
-
-	var ancestor string
-	var remoteAncestor string
-	go utils.Safe(func() {
-		defer wg.Done()
-
-		ancestor = opts.MainBranches.GetMergeBase(opts.RefName)
-		if opts.RefToShowDivergenceFrom != "" {
-			remoteAncestor = opts.MainBranches.GetMergeBase(opts.RefToShowDivergenceFrom)
+		if logErr == nil {
+			commits = append(commits, realCommits...)
 		}
 	})
 
-	passedFirstPushedCommit := false
-	// I can get this before
-	firstPushedCommit, err := self.getFirstPushedCommit(opts.RefForPushedStatus)
-	if err != nil {
-		// must have no upstream branch so we'll consider everything as pushed
-		passedFirstPushedCommit = true
+	var unmergedCommitHashes *set.Set[string]
+	var remoteUnmergedCommitHashes *set.Set[string]
+	mainBranches := opts.MainBranches.Get()
+
+	go utils.Safe(func() {
+		defer wg.Done()
+
+		if len(mainBranches) > 0 {
+			unmergedCommitHashes = self.getReachableHashes(opts.RefName, mainBranches)
+			if opts.RefToShowDivergenceFrom != "" {
+				remoteUnmergedCommitHashes = self.getReachableHashes(opts.RefToShowDivergenceFrom, mainBranches)
+			}
+		}
+	})
+
+	var unpushedCommitHashes *set.Set[string]
+	if opts.RefForPushedStatus != nil {
+		unpushedCommitHashes = self.getReachableHashes(opts.RefForPushedStatus.FullRefName(),
+			append([]string{opts.RefForPushedStatus.RefName() + "@{u}"}, mainBranches...))
 	}
 
 	wg.Wait()
@@ -123,19 +126,6 @@ func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit,
 		return nil, logErr
 	}
 
-	for _, commit := range commits {
-		if commit.Hash == firstPushedCommit {
-			passedFirstPushedCommit = true
-		}
-		if commit.Status != models.StatusRebasing {
-			if passedFirstPushedCommit {
-				commit.Status = models.StatusPushed
-			} else {
-				commit.Status = models.StatusUnpushed
-			}
-		}
-	}
-
 	if len(commits) == 0 {
 		return commits, nil
 	}
@@ -153,16 +143,16 @@ func (self *CommitLoader) GetCommits(opts GetCommitsOptions) ([]*models.Commit,
 			localSectionStart = len(commits)
 		}
 
-		setCommitMergedStatuses(remoteAncestor, commits[:localSectionStart])
-		setCommitMergedStatuses(ancestor, commits[localSectionStart:])
+		setCommitStatuses(unpushedCommitHashes, remoteUnmergedCommitHashes, commits[:localSectionStart])
+		setCommitStatuses(unpushedCommitHashes, unmergedCommitHashes, commits[localSectionStart:])
 	} else {
-		setCommitMergedStatuses(ancestor, commits)
+		setCommitStatuses(unpushedCommitHashes, unmergedCommitHashes, commits)
 	}
 
 	return commits, nil
 }
 
-func (self *CommitLoader) MergeRebasingCommits(commits []*models.Commit) ([]*models.Commit, error) {
+func (self *CommitLoader) MergeRebasingCommits(hashPool *utils.StringPool, commits []*models.Commit) ([]*models.Commit, error) {
 	// chances are we have as many commits as last time so we'll set the capacity to be the old length
 	result := make([]*models.Commit, 0, len(commits))
 	for i, commit := range commits {
@@ -172,24 +162,26 @@ func (self *CommitLoader) MergeRebasingCommits(commits []*models.Commit) ([]*mod
 		}
 	}
 
-	rebaseMode, err := self.getRebaseMode()
-	if err != nil {
-		return nil, err
+	workingTreeState := self.getWorkingTreeState()
+	addConflictedRebasingCommit := true
+	if workingTreeState.CherryPicking || workingTreeState.Reverting {
+		sequencerCommits, err := self.getHydratedSequencerCommits(hashPool, workingTreeState)
+		if err != nil {
+			return nil, err
+		}
+		result = append(sequencerCommits, result...)
+		addConflictedRebasingCommit = false
 	}
 
-	if rebaseMode == enums.REBASE_MODE_NONE {
-		// not in rebase mode so return original commits
-		return result, nil
+	if workingTreeState.Rebasing {
+		rebasingCommits, err := self.getHydratedRebasingCommits(hashPool, addConflictedRebasingCommit)
+		if err != nil {
+			return nil, err
+		}
+		if len(rebasingCommits) > 0 {
+			result = append(rebasingCommits, result...)
+		}
 	}
-
-	rebasingCommits, err := self.getHydratedRebasingCommits(rebaseMode)
-	if err != nil {
-		return nil, err
-	}
-	if len(rebasingCommits) > 0 {
-		result = append(rebasingCommits, result...)
-	}
-
 	return result, nil
 }
 
@@ -197,26 +189,37 @@ func (self *CommitLoader) MergeRebasingCommits(commits []*models.Commit) ([]*mod
 // then puts them into a commit object
 // example input:
 // 8ad01fe32fcc20f07bc6693f87aa4977c327f1e1|10 hours ago|Jesse Duffield| (HEAD -> master, tag: v0.15.2)|refresh commits when adding a tag
-func (self *CommitLoader) extractCommitFromLine(line string, showDivergence bool) *models.Commit {
+func (self *CommitLoader) extractCommitFromLine(hashPool *utils.StringPool, line string, showDivergence bool) *models.Commit {
 	split := strings.SplitN(line, "\x00", 8)
 
+	// Ensure we have the minimum required fields (at least 7 for basic functionality)
+	if len(split) < 7 {
+		self.Log.Warnf("Malformed git log line: expected at least 7 fields, got %d. Line: %s", len(split), line)
+		return nil
+	}
+
 	hash := split[0]
 	unixTimestamp := split[1]
 	authorName := split[2]
 	authorEmail := split[3]
-	extraInfo := strings.TrimSpace(split[4])
-	parentHashes := split[5]
+	parentHashes := split[4]
 	divergence := models.DivergenceNone
 	if showDivergence {
-		divergence = lo.Ternary(split[6] == "<", models.DivergenceLeft, models.DivergenceRight)
+		divergence = lo.Ternary(split[5] == "<", models.DivergenceLeft, models.DivergenceRight)
 	}
-	message := split[7]
+	extraInfo := strings.TrimSpace(split[6])
 
-	tags := []string{}
+	// message (and the \x00 before it) might not be present if extraInfo is extremely long
+	message := ""
+	if len(split) > 7 {
+		message = split[7]
+	}
+
+	var tags []string
 
 	if extraInfo != "" {
-		extraInfoFields := strings.Split(extraInfo, ",")
-		for _, extraInfoField := range extraInfoFields {
+		extraInfoFields := strings.SplitSeq(extraInfo, ",")
+		for extraInfoField := range extraInfoFields {
 			extraInfoField = strings.TrimSpace(extraInfoField)
 			re := regexp.MustCompile(`tag: (.+)`)
 			tagMatch := re.FindStringSubmatch(extraInfoField)
@@ -235,7 +238,7 @@ func (self *CommitLoader) extractCommitFromLine(line string, showDivergence bool
 		parents = strings.Split(parentHashes, " ")
 	}
 
-	return &models.Commit{
+	return models.NewCommit(hashPool, models.NewCommitOpts{
 		Hash:          hash,
 		Name:          message,
 		Tags:          tags,
@@ -245,18 +248,39 @@ func (self *CommitLoader) extractCommitFromLine(line string, showDivergence bool
 		AuthorEmail:   authorEmail,
 		Parents:       parents,
 		Divergence:    divergence,
-	}
+	})
 }
 
-func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode) ([]*models.Commit, error) {
-	commits := self.getRebasingCommits(rebaseMode)
+func (self *CommitLoader) getHydratedRebasingCommits(hashPool *utils.StringPool, addConflictingCommit bool) ([]*models.Commit, error) {
+	return self.getHydratedTodoCommits(hashPool, self.getRebasingCommits(hashPool, addConflictingCommit), false)
+}
 
-	if len(commits) == 0 {
+func (self *CommitLoader) getHydratedSequencerCommits(hashPool *utils.StringPool, workingTreeState models.WorkingTreeState) ([]*models.Commit, error) {
+	commits := self.getSequencerCommits(hashPool)
+	if len(commits) > 0 {
+		// If we have any commits in .git/sequencer/todo, then the last one of
+		// those is the conflicting one.
+		commits[len(commits)-1].Status = models.StatusConflicted
+	} else {
+		// For single-commit cherry-picks and reverts, git apparently doesn't
+		// use the sequencer; in that case, CHERRY_PICK_HEAD or REVERT_HEAD is
+		// our conflicting commit, so synthesize it here.
+		conflicedCommit := self.getConflictedSequencerCommit(hashPool, workingTreeState)
+		if conflicedCommit != nil {
+			commits = append(commits, conflicedCommit)
+		}
+	}
+
+	return self.getHydratedTodoCommits(hashPool, commits, true)
+}
+
+func (self *CommitLoader) getHydratedTodoCommits(hashPool *utils.StringPool, todoCommits []*models.Commit, todoFileHasShortHashes bool) ([]*models.Commit, error) {
+	if len(todoCommits) == 0 {
 		return nil, nil
 	}
 
-	commitHashes := lo.FilterMap(commits, func(commit *models.Commit, _ int) (string, bool) {
-		return commit.Hash, commit.Hash != ""
+	commitHashes := lo.FilterMap(todoCommits, func(commit *models.Commit, _ int) (string, bool) {
+		return commit.Hash(), commit.Hash() != ""
 	})
 
 	// note that we're not filtering these as we do non-rebasing commits just because
@@ -271,15 +295,18 @@ func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode
 
 	fullCommits := map[string]*models.Commit{}
 	err := cmdObj.RunAndProcessLines(func(line string) (bool, error) {
-		commit := self.extractCommitFromLine(line, false)
-		fullCommits[commit.Hash] = commit
+		if line == "" || line[0] != '+' {
+			return false, nil
+		}
+		commit := self.extractCommitFromLine(hashPool, line[1:], false)
+		fullCommits[commit.Hash()] = commit
 		return false, nil
 	})
 	if err != nil {
 		return nil, err
 	}
 
-	findFullCommit := lo.Ternary(self.version.IsOlderThan(2, 25, 2),
+	findFullCommit := lo.Ternary(todoFileHasShortHashes,
 		func(hash string) *models.Commit {
 			for s, c := range fullCommits {
 				if strings.HasPrefix(s, hash) {
@@ -292,12 +319,13 @@ func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode
 			return fullCommits[hash]
 		})
 
-	hydratedCommits := make([]*models.Commit, 0, len(commits))
-	for _, rebasingCommit := range commits {
-		if rebasingCommit.Hash == "" {
+	hydratedCommits := make([]*models.Commit, 0, len(todoCommits))
+	for _, rebasingCommit := range todoCommits {
+		if rebasingCommit.Hash() == "" {
 			hydratedCommits = append(hydratedCommits, rebasingCommit)
-		} else if commit := findFullCommit(rebasingCommit.Hash); commit != nil {
+		} else if commit := findFullCommit(rebasingCommit.Hash()); commit != nil {
 			commit.Action = rebasingCommit.Action
+			commit.ActionFlag = rebasingCommit.ActionFlag
 			commit.Status = rebasingCommit.Status
 			hydratedCommits = append(hydratedCommits, commit)
 		}
@@ -310,11 +338,7 @@ func (self *CommitLoader) getHydratedRebasingCommits(rebaseMode enums.RebaseMode
 // git-rebase-todo example:
 // pick ac446ae94ee560bdb8d1d057278657b251aaef17 ac446ae
 // pick afb893148791a2fbd8091aeb81deba4930c73031 afb8931
-func (self *CommitLoader) getRebasingCommits(rebaseMode enums.RebaseMode) []*models.Commit {
-	if rebaseMode != enums.REBASE_MODE_INTERACTIVE {
-		return nil
-	}
-
+func (self *CommitLoader) getRebasingCommits(hashPool *utils.StringPool, addConflictingCommit bool) []*models.Commit {
 	bytesContent, err := self.readFile(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/git-rebase-todo"))
 	if err != nil {
 		self.Log.Error(fmt.Sprintf("error occurred reading git-rebase-todo: %s", err.Error()))
@@ -332,13 +356,10 @@ func (self *CommitLoader) getRebasingCommits(rebaseMode enums.RebaseMode) []*mod
 
 	// See if the current commit couldn't be applied because it conflicted; if
 	// so, add a fake entry for it
-	if conflictedCommitHash := self.getConflictedCommit(todos); conflictedCommitHash != "" {
-		commits = append(commits, &models.Commit{
-			Hash:   conflictedCommitHash,
-			Name:   "",
-			Status: models.StatusRebasing,
-			Action: models.ActionConflict,
-		})
+	if addConflictingCommit {
+		if conflictedCommit := self.getConflictedCommit(hashPool, todos); conflictedCommit != nil {
+			commits = append(commits, conflictedCommit)
+		}
 	}
 
 	for _, t := range todos {
@@ -350,47 +371,46 @@ func (self *CommitLoader) getRebasingCommits(rebaseMode enums.RebaseMode) []*mod
 			// Command does not have a commit associated, skip
 			continue
 		}
-		commits = utils.Prepend(commits, &models.Commit{
-			Hash:   t.Commit,
-			Name:   t.Msg,
-			Status: models.StatusRebasing,
-			Action: t.Command,
-		})
+		commits = utils.Prepend(commits, models.NewCommit(hashPool, models.NewCommitOpts{
+			Hash:       t.Commit,
+			Name:       t.Msg,
+			Status:     models.StatusRebasing,
+			Action:     t.Command,
+			ActionFlag: t.Flag,
+		}))
 	}
 
 	return commits
 }
 
-func (self *CommitLoader) getConflictedCommit(todos []todo.Todo) string {
+func (self *CommitLoader) getConflictedCommit(hashPool *utils.StringPool, todos []todo.Todo) *models.Commit {
 	bytesContent, err := self.readFile(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/done"))
 	if err != nil {
 		self.Log.Error(fmt.Sprintf("error occurred reading rebase-merge/done: %s", err.Error()))
-		return ""
+		return nil
 	}
 
 	doneTodos, err := todo.Parse(bytes.NewBuffer(bytesContent), self.config.GetCoreCommentChar())
 	if err != nil {
 		self.Log.Error(fmt.Sprintf("error occurred while parsing rebase-merge/done file: %s", err.Error()))
-		return ""
+		return nil
 	}
 
-	amendFileExists := false
-	if _, err := os.Stat(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/amend")); err == nil {
-		amendFileExists = true
-	}
+	amendFileExists, _ := self.os.FileExists(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/amend"))
+	messageFileExists, _ := self.os.FileExists(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "rebase-merge/message"))
 
-	return self.getConflictedCommitImpl(todos, doneTodos, amendFileExists)
+	return self.getConflictedCommitImpl(hashPool, todos, doneTodos, amendFileExists, messageFileExists)
 }
 
-func (self *CommitLoader) getConflictedCommitImpl(todos []todo.Todo, doneTodos []todo.Todo, amendFileExists bool) string {
+func (self *CommitLoader) getConflictedCommitImpl(hashPool *utils.StringPool, todos []todo.Todo, doneTodos []todo.Todo, amendFileExists bool, messageFileExists bool) *models.Commit {
 	// Should never be possible, but just to be safe:
 	if len(doneTodos) == 0 {
 		self.Log.Error("no done entries in rebase-merge/done file")
-		return ""
+		return nil
 	}
 	lastTodo := doneTodos[len(doneTodos)-1]
 	if lastTodo.Command == todo.Break || lastTodo.Command == todo.Exec || lastTodo.Command == todo.Reword {
-		return ""
+		return nil
 	}
 
 	// In certain cases, git reschedules commands that failed. One example is if
@@ -401,7 +421,7 @@ func (self *CommitLoader) getConflictedCommitImpl(todos []todo.Todo, doneTodos [
 	// same, the command was rescheduled.
 	if len(doneTodos) > 0 && len(todos) > 0 && doneTodos[len(doneTodos)-1] == todos[0] {
 		// Command was rescheduled, no need to display it
-		return ""
+		return nil
 	}
 
 	// Older versions of git have a bug whereby, if a command is rescheduled,
@@ -426,79 +446,140 @@ func (self *CommitLoader) getConflictedCommitImpl(todos []todo.Todo, doneTodos [
 	if len(doneTodos) >= 3 && len(todos) > 0 && doneTodos[len(doneTodos)-2] == todos[0] &&
 		doneTodos[len(doneTodos)-1] == doneTodos[len(doneTodos)-3] {
 		// Command was rescheduled, no need to display it
-		return ""
+		return nil
 	}
 
 	if lastTodo.Command == todo.Edit {
 		if amendFileExists {
 			// Special case for "edit": if the "amend" file exists, the "edit"
 			// command was successful, otherwise it wasn't
-			return ""
+			return nil
+		}
+
+		if !messageFileExists {
+			// As an additional check, see if the "message" file exists; if it
+			// doesn't, it must be because a multi-commit cherry-pick or revert
+			// was performed in the meantime, which deleted both the amend file
+			// and the message file.
+			return nil
 		}
 	}
 
 	// I don't think this is ever possible, but again, just to be safe:
 	if lastTodo.Commit == "" {
 		self.Log.Error("last command in rebase-merge/done file doesn't have a commit")
-		return ""
+		return nil
 	}
 
 	// Any other todo that has a commit associated with it must have failed with
 	// a conflict, otherwise we wouldn't have stopped the rebase:
-	return lastTodo.Commit
+	return models.NewCommit(hashPool, models.NewCommitOpts{
+		Hash:   lastTodo.Commit,
+		Action: lastTodo.Command,
+		Status: models.StatusConflicted,
+	})
 }
 
-func setCommitMergedStatuses(ancestor string, commits []*models.Commit) {
-	if ancestor == "" {
-		return
+func (self *CommitLoader) getSequencerCommits(hashPool *utils.StringPool) []*models.Commit {
+	bytesContent, err := self.readFile(filepath.Join(self.repoPaths.WorktreeGitDirPath(), "sequencer/todo"))
+	if err != nil {
+		self.Log.Error(fmt.Sprintf("error occurred reading sequencer/todo: %s", err.Error()))
+		// we assume an error means the file doesn't exist so we just return
+		return nil
 	}
 
-	passedAncestor := false
-	for i, commit := range commits {
-		// some commits aren't really commits and don't have hashes, such as the update-ref todo
-		if commit.Hash != "" && strings.HasPrefix(ancestor, commit.Hash) {
-			passedAncestor = true
-		}
-		if commit.Status != models.StatusPushed && commit.Status != models.StatusUnpushed {
+	commits := []*models.Commit{}
+
+	todos, err := todo.Parse(bytes.NewBuffer(bytesContent), self.config.GetCoreCommentChar())
+	if err != nil {
+		self.Log.Error(fmt.Sprintf("error occurred while parsing sequencer/todo file: %s", err.Error()))
+		return nil
+	}
+
+	for _, t := range todos {
+		if t.Commit == "" {
+			// Command does not have a commit associated, skip
 			continue
 		}
-		if passedAncestor {
+		commits = utils.Prepend(commits, models.NewCommit(hashPool, models.NewCommitOpts{
+			Hash:   t.Commit,
+			Name:   t.Msg,
+			Status: models.StatusCherryPickingOrReverting,
+			Action: t.Command,
+		}))
+	}
+
+	return commits
+}
+
+func (self *CommitLoader) getConflictedSequencerCommit(hashPool *utils.StringPool, workingTreeState models.WorkingTreeState) *models.Commit {
+	var shaFile string
+	var action todo.TodoCommand
+	if workingTreeState.CherryPicking {
+		shaFile = "CHERRY_PICK_HEAD"
+		action = todo.Pick
+	} else if workingTreeState.Reverting {
+		shaFile = "REVERT_HEAD"
+		action = todo.Revert
+	} else {
+		return nil
+	}
+	bytesContent, err := self.readFile(filepath.Join(self.repoPaths.WorktreeGitDirPath(), shaFile))
+	if err != nil {
+		self.Log.Error(fmt.Sprintf("error occurred reading %s: %s", shaFile, err.Error()))
+		// we assume an error means the file doesn't exist so we just return
+		return nil
+	}
+	lines := strings.Split(string(bytesContent), "\n")
+	if len(lines) == 0 {
+		return nil
+	}
+	return models.NewCommit(hashPool, models.NewCommitOpts{
+		Hash:   lines[0],
+		Status: models.StatusConflicted,
+		Action: action,
+	})
+}
+
+func setCommitStatuses(unpushedCommitHashes *set.Set[string], unmergedCommitHashes *set.Set[string], commits []*models.Commit) {
+	for i, commit := range commits {
+		if commit.IsTODO() {
+			continue
+		}
+
+		if unmergedCommitHashes == nil || unmergedCommitHashes.Includes(commit.Hash()) {
+			if unpushedCommitHashes != nil && unpushedCommitHashes.Includes(commit.Hash()) {
+				commits[i].Status = models.StatusUnpushed
+			} else {
+				commits[i].Status = models.StatusPushed
+			}
+		} else {
 			commits[i].Status = models.StatusMerged
 		}
 	}
 }
 
-func ignoringWarnings(commandOutput string) string {
-	trimmedOutput := strings.TrimSpace(commandOutput)
-	split := strings.Split(trimmedOutput, "\n")
-	// need to get last line in case the first line is a warning about how the error is ambiguous.
-	// At some point we should find a way to make it unambiguous
-	lastLine := split[len(split)-1]
-
-	return lastLine
-}
-
-// getFirstPushedCommit returns the first commit hash which has been pushed to the ref's upstream.
-// all commits above this are deemed unpushed and marked as such.
-func (self *CommitLoader) getFirstPushedCommit(refName string) (string, error) {
-	output, err := self.cmd.New(
-		NewGitCmd("merge-base").
+func (self *CommitLoader) getReachableHashes(refName string, notRefNames []string) *set.Set[string] {
+	output, _, err := self.cmd.New(
+		NewGitCmd("rev-list").
 			Arg(refName).
-			Arg(strings.TrimPrefix(refName, "refs/heads/") + "@{u}").
+			Arg(lo.Map(notRefNames, func(name string, _ int) string {
+				return "^" + name
+			})...).
 			ToArgv(),
 	).
 		DontLog().
-		RunWithOutput()
+		RunWithOutputs()
 	if err != nil {
-		return "", err
+		return set.New[string]()
 	}
 
-	return ignoringWarnings(output), nil
+	return set.NewFromSlice(utils.SplitLines(output))
 }
 
-// getLog gets the git log.
-func (self *CommitLoader) getLogCmd(opts GetCommitsOptions) oscommands.ICmdObj {
-	gitLogOrder := self.AppState.GitLogOrder
+// getLogCmd gets the git log.
+func (self *CommitLoader) getLogCmd(opts GetCommitsOptions) *oscommands.CmdObj {
+	gitLogOrder := self.UserConfig().Git.Log.Order
 
 	refSpec := opts.RefName
 	if opts.RefToShowDivergenceFrom != "" {
@@ -514,7 +595,7 @@ func (self *CommitLoader) getLogCmd(opts GetCommitsOptions) oscommands.ICmdObj {
 		Arg("--abbrev=40").
 		ArgIf(opts.FilterAuthor != "", "--author="+opts.FilterAuthor).
 		ArgIf(opts.Limit, "-300").
-		ArgIf(opts.FilterPath != "", "--follow").
+		ArgIf(opts.FilterPath != "", "--follow", "--name-status").
 		Arg("--no-show-signature").
 		ArgIf(opts.RefToShowDivergenceFrom != "", "--left-right").
 		Arg("--").
@@ -524,4 +605,4 @@ func (self *CommitLoader) getLogCmd(opts GetCommitsOptions) oscommands.ICmdObj {
 	return self.cmd.New(cmdArgs).DontLog()
 }
 
-const prettyFormat = `--pretty=format:%H%x00%at%x00%aN%x00%ae%x00%D%x00%p%x00%m%x00%s`
+const prettyFormat = `--pretty=format:+%H%x00%at%x00%aN%x00%ae%x00%P%x00%m%x00%D%x00%s`
