@@ -35,6 +35,8 @@ func NewRebaseCommands(
 }
 
 func (self *RebaseCommands) RewordCommit(commits []*models.Commit, index int, summary string, description string) error {
+	// This check is currently unreachable (handled in LocalCommitsController.reword),
+	// but kept as a safeguard in case this method is used elsewhere.
 	if self.config.NeedsGpgSubprocessForCommit() {
 		return errors.New(self.Tr.DisabledForGPG)
 	}
@@ -137,7 +139,7 @@ func (self *RebaseCommands) MoveCommitsUp(commits []*models.Commit, startIdx int
 	}).Run()
 }
 
-func (self *RebaseCommands) InteractiveRebase(commits []*models.Commit, startIdx int, endIdx int, action todo.TodoCommand) error {
+func (self *RebaseCommands) InteractiveRebase(commits []*models.Commit, startIdx int, endIdx int, action todo.TodoCommand, flag string) error {
 	baseIndex := endIdx + 1
 	if action == todo.Squash || action == todo.Fixup {
 		baseIndex++
@@ -149,6 +151,7 @@ func (self *RebaseCommands) InteractiveRebase(commits []*models.Commit, startIdx
 		return daemon.ChangeTodoAction{
 			Hash:      commit.Hash(),
 			NewAction: action,
+			Flag:      flag,
 		}, !commit.IsMerge()
 	})
 
@@ -216,7 +219,7 @@ func (self *RebaseCommands) PrepareInteractiveRebaseCommand(opts PrepareInteract
 		Arg("--interactive").
 		Arg("--autostash").
 		Arg("--keep-empty").
-		ArgIf(opts.keepCommitsThatBecomeEmpty && self.version.IsAtLeast(2, 26, 0), "--empty=keep").
+		ArgIf(opts.keepCommitsThatBecomeEmpty, "--empty=keep").
 		Arg("--no-autosquash").
 		Arg("--rebase-merges").
 		ArgIf(opts.onto != "", "--onto", opts.onto).
@@ -242,8 +245,9 @@ func (self *RebaseCommands) PrepareInteractiveRebaseCommand(opts PrepareInteract
 
 	cmdObj.AddEnvVars(
 		"DEBUG="+debug,
-		"LANG=en_US.UTF-8",   // Force using EN as language
-		"LC_ALL=en_US.UTF-8", // Force using EN as language
+		"LANG=C",        // Force using English language
+		"LC_ALL=C",      // Force using English language
+		"LC_MESSAGES=C", // Force using English language
 		"GIT_SEQUENCE_EDITOR="+gitSequenceEditor,
 	)
 
@@ -275,8 +279,9 @@ func (self *RebaseCommands) GitRebaseEditTodo(todosFileContent []byte) error {
 
 	cmdObj.AddEnvVars(
 		"DEBUG="+debug,
-		"LANG=en_US.UTF-8",   // Force using EN as language
-		"LC_ALL=en_US.UTF-8", // Force using EN as language
+		"LANG=C",        // Force using English language
+		"LC_ALL=C",      // Force using English language
+		"LC_MESSAGES=C", // Force using English language
 		"GIT_EDITOR="+ex,
 		"GIT_SEQUENCE_EDITOR="+ex,
 	)
@@ -325,17 +330,17 @@ func (self *RebaseCommands) MoveFixupCommitDown(commits []*models.Commit, target
 func todoFromCommit(commit *models.Commit) utils.Todo {
 	if commit.Action == todo.UpdateRef {
 		return utils.Todo{Ref: commit.Name}
-	} else {
-		return utils.Todo{Hash: commit.Hash()}
 	}
+	return utils.Todo{Hash: commit.Hash()}
 }
 
 // Sets the action for the given commits in the git-rebase-todo file
-func (self *RebaseCommands) EditRebaseTodo(commits []*models.Commit, action todo.TodoCommand) error {
+func (self *RebaseCommands) EditRebaseTodo(commits []*models.Commit, action todo.TodoCommand, flag string) error {
 	commitsWithAction := lo.Map(commits, func(commit *models.Commit, _ int) utils.TodoChange {
 		return utils.TodoChange{
 			Hash:      commit.Hash(),
 			NewAction: action,
+			Flag:      flag,
 		}
 	})
 
@@ -410,9 +415,9 @@ func (self *RebaseCommands) BeginInteractiveRebaseForCommit(
 			instruction:                daemon.NewInsertBreakInstruction(),
 			keepCommitsThatBecomeEmpty: keepCommitsThatBecomeEmpty,
 		}).Run()
-	} else {
-		return self.BeginInteractiveRebaseForCommitRange(commits, commitIndex, commitIndex, keepCommitsThatBecomeEmpty)
 	}
+
+	return self.BeginInteractiveRebaseForCommitRange(commits, commitIndex, commitIndex, keepCommitsThatBecomeEmpty)
 }
 
 func (self *RebaseCommands) BeginInteractiveRebaseForCommitRange(
@@ -518,10 +523,17 @@ func (self *RebaseCommands) DiscardOldFileChanges(commits []*models.Commit, comm
 	}
 
 	for _, filePath := range filePaths {
-		// check if file exists in previous commit (this command returns an error if the file doesn't exist)
-		cmdArgs := NewGitCmd("cat-file").Arg("-e", "HEAD^:"+filePath).ToArgv()
-
-		if err := self.cmd.New(cmdArgs).Run(); err != nil {
+		doesFileExistInPreviousCommit := false
+		if commitIndex < len(commits)-1 {
+			// check if file exists in previous commit (this command returns an empty string if the file doesn't exist)
+			cmdArgs := NewGitCmd("ls-tree").Arg("--name-only", "HEAD^", "--", filePath).ToArgv()
+			output, err := self.cmd.New(cmdArgs).DontLog().RunWithOutput()
+			if err != nil {
+				return err
+			}
+			doesFileExistInPreviousCommit = strings.TrimRight(output, "\n") == filePath
+		}
+		if !doesFileExistInPreviousCommit {
 			if err := self.os.Remove(filePath); err != nil {
 				return err
 			}
@@ -572,7 +584,7 @@ func getBaseHashOrRoot(commits []*models.Commit, index int) string {
 	// at time of writing)
 	if index < len(commits) {
 		return commits[index].Hash()
-	} else {
-		return "--root"
 	}
+
+	return "--root"
 }
