@@ -551,6 +551,7 @@ type KeybindingCommitsConfig struct {
 	RenameCommitWithEditor         string `yaml:"renameCommitWithEditor"`
 	ViewResetOptions               string `yaml:"viewResetOptions"`
 	MarkCommitAsFixup              string `yaml:"markCommitAsFixup"`
+	SetFixupMessage                string `yaml:"setFixupMessage"`
 	CreateFixupCommit              string `yaml:"createFixupCommit"`
 	SquashAboveCommits             string `yaml:"squashAboveCommits"`
 	MoveDownCommit                 string `yaml:"moveDownCommit"`
@@ -1008,6 +1009,7 @@ func GetDefaultConfig() *UserConfig {
 				RenameCommitWithEditor:         "R",
 				ViewResetOptions:               "g",
 				MarkCommitAsFixup:              "f",
+				SetFixupMessage:                "c",
 				CreateFixupCommit:              "F",
 				SquashAboveCommits:             "S",
 				MoveDownCommit:                 "<c-j>",
