@@ -28,7 +28,7 @@ type pipeSetCacheKey struct {
 }
 
 var (
-	pipeSetCache = make(map[pipeSetCacheKey][][]*graph.Pipe)
+	pipeSetCache = make(map[pipeSetCacheKey][][]graph.Pipe)
 	mutex        deadlock.Mutex
 )
 
@@ -51,12 +51,11 @@ func GetCommitListDisplayStrings(
 	shortTimeFormat string,
 	now time.Time,
 	parseEmoji bool,
-	selectedCommitHash string,
+	selectedCommitHashPtr *string,
 	startIdx int,
 	endIdx int,
 	showGraph bool,
 	bisectInfo *git_commands.BisectInfo,
-	showYouAreHereLabel bool,
 ) [][]string {
 	mutex.Lock()
 	defer mutex.Unlock()
@@ -103,7 +102,7 @@ func GetCommitListDisplayStrings(
 					graphLines := graph.RenderAux(
 						graphPipeSets,
 						graphCommits,
-						selectedCommitHash,
+						selectedCommitHashPtr,
 					)
 					allGraphLines = append(allGraphLines, graphLines...)
 				}
@@ -120,7 +119,7 @@ func GetCommitListDisplayStrings(
 					graphLines := graph.RenderAux(
 						graphPipeSets,
 						graphCommits,
-						selectedCommitHash,
+						selectedCommitHashPtr,
 					)
 					allGraphLines = append(allGraphLines, graphLines...)
 				}
@@ -141,14 +140,13 @@ func GetCommitListDisplayStrings(
 			graphLines := graph.RenderAux(
 				graphPipeSets,
 				graphCommits,
-				selectedCommitHash,
+				selectedCommitHashPtr,
 			)
 			getGraphLine = func(idx int) string {
 				if idx >= graphOffset {
 					return graphLines[idx-graphOffset]
-				} else {
-					return ""
 				}
+				return ""
 			}
 		}
 	} else {
@@ -176,7 +174,7 @@ func GetCommitListDisplayStrings(
 					!lo.Contains(common.UserConfig().Git.MainBranches, b.Name) &&
 					// Don't show a marker for the head commit unless the
 					// rebase.updateRefs config is on
-					(hasRebaseUpdateRefsConfig || b.CommitHash != commits[0].Hash)
+					(hasRebaseUpdateRefsConfig || b.CommitHash != commits[0].Hash())
 		}))
 
 	lines := make([][]string, 0, len(filteredCommits))
@@ -184,13 +182,8 @@ func GetCommitListDisplayStrings(
 	willBeRebased := markedBaseCommit == ""
 	for i, commit := range filteredCommits {
 		unfilteredIdx := i + startIdx
-		bisectStatus = getBisectStatus(unfilteredIdx, commit.Hash, bisectInfo, bisectBounds)
-		isYouAreHereCommit := false
-		if showYouAreHereLabel && (commit.Action == models.ActionConflict || unfilteredIdx == rebaseOffset) {
-			isYouAreHereCommit = true
-			showYouAreHereLabel = false
-		}
-		isMarkedBaseCommit := commit.Hash != "" && commit.Hash == markedBaseCommit
+		bisectStatus = getBisectStatus(unfilteredIdx, commit.Hash(), bisectInfo, bisectBounds)
+		isMarkedBaseCommit := commit.Hash() != "" && commit.Hash() == markedBaseCommit
 		if isMarkedBaseCommit {
 			willBeRebased = true
 		}
@@ -211,7 +204,6 @@ func GetCommitListDisplayStrings(
 			fullDescription,
 			bisectStatus,
 			bisectInfo,
-			isYouAreHereCommit,
 		))
 	}
 	return lines
@@ -225,11 +217,11 @@ func getbisectBounds(commits []*models.Commit, bisectInfo *git_commands.BisectIn
 	bisectBounds := &bisectBounds{}
 
 	for i, commit := range commits {
-		if commit.Hash == bisectInfo.GetNewHash() {
+		if commit.Hash() == bisectInfo.GetNewHash() {
 			bisectBounds.newIndex = i
 		}
 
-		status, ok := bisectInfo.Status(commit.Hash)
+		status, ok := bisectInfo.Status(commit.Hash())
 		if ok && status == git_commands.BisectStatusOld {
 			bisectBounds.oldIndex = i
 			return bisectBounds
@@ -252,11 +244,11 @@ func indexOfFirstNonTODOCommit(commits []*models.Commit) int {
 	return 0
 }
 
-func loadPipesets(commits []*models.Commit) [][]*graph.Pipe {
+func loadPipesets(commits []*models.Commit) [][]graph.Pipe {
 	// given that our cache key is a commit hash and a commit count, it's very important that we don't actually try to render pipes
 	// when dealing with things like filtered commits.
 	cacheKey := pipeSetCacheKey{
-		commitHash:  commits[0].Hash,
+		commitHash:  commits[0].Hash(),
 		commitCount: len(commits),
 		divergence:  commits[0].Divergence,
 	}
@@ -265,7 +257,7 @@ func loadPipesets(commits []*models.Commit) [][]*graph.Pipe {
 	if !ok {
 		// pipe sets are unique to a commit head. and a commit count. Sometimes we haven't loaded everything for that.
 		// so let's just cache it based on that.
-		getStyle := func(commit *models.Commit) style.TextStyle {
+		getStyle := func(commit *models.Commit) *style.TextStyle {
 			return authors.AuthorStyle(commit.AuthorName)
 		}
 		pipeSets = graph.GetPipeSets(commits, getStyle)
@@ -312,9 +304,8 @@ func getBisectStatus(index int, commitHash string, bisectInfo *git_commands.Bise
 	} else {
 		if bisectBounds != nil && index >= bisectBounds.newIndex && index <= bisectBounds.oldIndex {
 			return BisectStatusCandidate
-		} else {
-			return BisectStatusNone
 		}
+		return BisectStatusNone
 	}
 
 	// should never land here
@@ -364,17 +355,16 @@ func displayCommit(
 	fullDescription bool,
 	bisectStatus BisectStatus,
 	bisectInfo *git_commands.BisectInfo,
-	isYouAreHereCommit bool,
 ) []string {
 	bisectString := getBisectStatusText(bisectStatus, bisectInfo)
 
 	hashString := ""
 	hashColor := getHashColor(commit, diffName, cherryPickedCommitHashSet, bisectStatus, bisectInfo)
 	hashLength := common.UserConfig().Gui.CommitHashLength
-	if hashLength >= len(commit.Hash) {
-		hashString = hashColor.Sprint(commit.Hash)
+	if hashLength >= len(commit.Hash()) {
+		hashString = hashColor.Sprint(commit.Hash())
 	} else if hashLength > 0 {
-		hashString = hashColor.Sprint(commit.Hash[:hashLength])
+		hashString = hashColor.Sprint(commit.Hash()[:hashLength])
 	} else if !icons.IsIconEnabled() { // hashLength <= 0
 		hashString = hashColor.Sprint("*")
 	}
@@ -395,8 +385,12 @@ func displayCommit(
 
 	actionString := ""
 	if commit.Action != models.ActionNone {
-		todoString := lo.Ternary(commit.Action == models.ActionConflict, "conflict", commit.Action.String())
-		actionString = actionColorMap(commit.Action).Sprint(todoString) + " "
+		actionStr := commit.Action.String()
+		// Only show the flag for fixup commands (where -C changes the meaning)
+		if commit.ActionFlag != "" && commit.Action == todo.Fixup {
+			actionStr += " " + commit.ActionFlag
+		}
+		actionString = actionColorMap(commit.Action, commit.Status).Sprint(actionStr)
 	}
 
 	tagString := ""
@@ -409,7 +403,7 @@ func displayCommit(
 			tagString = theme.DiffTerminalColor.SetBold().Sprint(strings.Join(commit.Tags, " ")) + " "
 		}
 
-		if branchHeadsToVisualize.Includes(commit.Hash) &&
+		if branchHeadsToVisualize.Includes(commit.Hash()) &&
 			// Don't show branch head on commits that are already merged to a main branch
 			commit.Status != models.StatusMerged &&
 			// Don't show branch head on a "pick" todo if the rebase.updateRefs config is on
@@ -428,9 +422,8 @@ func displayCommit(
 	}
 
 	mark := ""
-	if isYouAreHereCommit {
-		color := lo.Ternary(commit.Action == models.ActionConflict, style.FgRed, style.FgYellow)
-		youAreHere := color.Sprintf("<-- %s ---", common.Tr.YouAreHere)
+	if commit.Status == models.StatusConflicted {
+		youAreHere := style.FgRed.Sprintf("<-- %s ---", common.Tr.ConflictLabel)
 		mark = fmt.Sprintf("%s ", youAreHere)
 	} else if isMarkedBaseCommit {
 		rebaseFromHere := style.FgYellow.Sprint(common.Tr.MarkedCommitMarker)
@@ -492,7 +485,7 @@ func getHashColor(
 		return getBisectStatusColor(bisectStatus)
 	}
 
-	diffed := commit.Hash != "" && commit.Hash == diffName
+	diffed := commit.Hash() != "" && commit.Hash() == diffName
 	hashColor := theme.DefaultTextColor
 	switch commit.Status {
 	case models.StatusUnpushed:
@@ -501,7 +494,7 @@ func getHashColor(
 		hashColor = style.FgYellow
 	case models.StatusMerged:
 		hashColor = style.FgGreen
-	case models.StatusRebasing:
+	case models.StatusRebasing, models.StatusCherryPickingOrReverting, models.StatusConflicted:
 		hashColor = style.FgBlue
 	case models.StatusReflog:
 		hashColor = style.FgBlue
@@ -510,7 +503,7 @@ func getHashColor(
 
 	if diffed {
 		hashColor = theme.DiffTerminalColor
-	} else if cherryPickedCommitHashSet.Includes(commit.Hash) {
+	} else if cherryPickedCommitHashSet.Includes(commit.Hash()) {
 		hashColor = theme.CherryPickedCommitTextStyle
 	} else if commit.Divergence == models.DivergenceRight && commit.Status != models.StatusMerged {
 		hashColor = style.FgBlue
@@ -519,7 +512,11 @@ func getHashColor(
 	return hashColor
 }
 
-func actionColorMap(action todo.TodoCommand) style.TextStyle {
+func actionColorMap(action todo.TodoCommand, status models.CommitStatus) style.TextStyle {
+	if status == models.StatusConflicted {
+		return style.FgRed
+	}
+
 	switch action {
 	case todo.Pick:
 		return style.FgCyan
@@ -529,8 +526,6 @@ func actionColorMap(action todo.TodoCommand) style.TextStyle {
 		return style.FgGreen
 	case todo.Fixup:
 		return style.FgMagenta
-	case models.ActionConflict:
-		return style.FgRed
 	default:
 		return style.FgYellow
 	}
