@@ -9,6 +9,7 @@ import (
 	"github.com/jesseduffield/gocui"
 	"github.com/jesseduffield/lazygit/pkg/config"
 	"github.com/jesseduffield/lazygit/pkg/gui/controllers/helpers"
+	"github.com/jesseduffield/lazygit/pkg/gui/keybindings"
 	"github.com/jesseduffield/lazygit/pkg/gui/style"
 	"github.com/jesseduffield/lazygit/pkg/gui/types"
 	"github.com/jesseduffield/lazygit/pkg/utils"
@@ -125,6 +126,8 @@ func (self *HandlerCreator) inputPrompt(prompt *config.CustomCommandPrompt, wrap
 		HandleConfirm: func(str string) error {
 			return wrappedF(str)
 		},
+		AllowEmptyInput:    true,
+		PreserveWhitespace: true,
 	})
 
 	return nil
@@ -201,6 +204,7 @@ func (self *HandlerCreator) menuPrompt(prompt *config.CustomCommandPrompt, wrapp
 			OnPress: func() error {
 				return wrappedF(option.Value)
 			},
+			Key: keybindings.GetKey(option.Key),
 		}
 	})
 
@@ -282,9 +286,7 @@ func (self *HandlerCreator) finalHandler(customCommand config.CustomCommand, ses
 		}
 		output, err := cmdObj.RunWithOutput()
 
-		if refreshErr := self.c.Refresh(types.RefreshOptions{Mode: types.ASYNC}); err != nil {
-			self.c.Log.Error(refreshErr)
-		}
+		self.c.Refresh(types.RefreshOptions{Mode: types.ASYNC})
 
 		if err != nil {
 			if customCommand.After != nil && customCommand.After.CheckForConflicts {
