@@ -68,6 +68,7 @@ func (gui *Gui) orderedViewNameMappings() []viewNameMapping {
 		{viewPtr: &gui.Views.Menu, name: "menu"},
 		{viewPtr: &gui.Views.Suggestions, name: "suggestions"},
 		{viewPtr: &gui.Views.Confirmation, name: "confirmation"},
+		{viewPtr: &gui.Views.Prompt, name: "prompt"},
 		{viewPtr: &gui.Views.Tooltip, name: "tooltip"},
 
 		// this guy will cover everything else when it appears
@@ -101,7 +102,6 @@ func (gui *Gui) createAllViews() error {
 
 	for _, view := range []*gocui.View{gui.Views.Main, gui.Views.Secondary, gui.Views.Staging, gui.Views.StagingSecondary, gui.Views.PatchBuilding, gui.Views.PatchBuildingSecondary, gui.Views.MergeConflicts} {
 		view.Wrap = true
-		view.IgnoreCarriageReturns = true
 		view.UnderlineHyperLinksOnlyOnHover = true
 		view.AutoRenderHyperLinks = true
 	}
@@ -127,9 +127,14 @@ func (gui *Gui) createAllViews() error {
 	gui.Views.CommitDescription.Editor = gocui.EditorFunc(gui.commitDescriptionEditor)
 
 	gui.Views.Confirmation.Visible = false
-	gui.Views.Confirmation.Editor = gocui.EditorFunc(gui.promptEditor)
+	gui.Views.Confirmation.Wrap = true
 	gui.Views.Confirmation.AutoRenderHyperLinks = true
 
+	gui.Views.Prompt.Visible = false
+	gui.Views.Prompt.Wrap = false // We don't want wrapping in one-line prompts
+	gui.Views.Prompt.Editable = true
+	gui.Views.Prompt.Editor = gocui.EditorFunc(gui.promptEditor)
+
 	gui.Views.Suggestions.Visible = false
 
 	gui.Views.Menu.Visible = false
