@@ -6,25 +6,20 @@ import (
 	"fmt"
 	"os"
 	"slices"
-	"strings"
 
 	"github.com/samber/lo"
 	"github.com/stefanhaller/git-todo-parser/todo"
 )
 
 type Todo struct {
-	Hash   string // for todos that have one, e.g. pick, drop, fixup, etc.
-	Ref    string // for update-ref todos
-	Action todo.TodoCommand
+	Hash string // for todos that have one, e.g. pick, drop, fixup, etc.
+	Ref  string // for update-ref todos
 }
 
-// In order to change a TODO in git-rebase-todo, we need to specify the old action,
-// because sometimes the same hash appears multiple times in the file (e.g. in a pick
-// and later in a merge)
 type TodoChange struct {
 	Hash      string
-	OldAction todo.TodoCommand
 	NewAction todo.TodoCommand
+	Flag      string
 }
 
 // Read a git-rebase-todo file, change the actions for the given commits,
@@ -40,9 +35,10 @@ func EditRebaseTodo(filePath string, changes []TodoChange, commentChar byte) err
 		t := &todos[i]
 		// This is a nested loop, but it's ok because the number of todos should be small
 		for _, change := range changes {
-			if t.Command == change.OldAction && equalHash(t.Commit, change.Hash) {
+			if equalHash(t.Commit, change.Hash) {
 				matchCount++
 				t.Command = change.NewAction
+				t.Flag = change.Flag
 			}
 		}
 	}
@@ -56,18 +52,18 @@ func EditRebaseTodo(filePath string, changes []TodoChange, commentChar byte) err
 }
 
 func equalHash(a, b string) bool {
-	return strings.HasPrefix(a, b) || strings.HasPrefix(b, a)
+	if len(a) == 0 && len(b) == 0 {
+		return true
+	}
+
+	commonLength := min(len(a), len(b))
+	return commonLength > 0 && a[:commonLength] == b[:commonLength]
 }
 
 func findTodo(todos []todo.Todo, todoToFind Todo) (int, bool) {
 	_, idx, ok := lo.FindIndexOf(todos, func(t todo.Todo) bool {
-		// Comparing just the hash is not enough; we need to compare both the
-		// action and the hash, as the hash could appear multiple times (e.g. in a
-		// pick and later in a merge). For update-ref todos we also must compare
-		// the Ref.
-		return t.Command == todoToFind.Action &&
-			equalHash(t.Commit, todoToFind.Hash) &&
-			t.Ref == todoToFind.Ref
+		// For update-ref todos we also must compare the Ref (they have an empty hash)
+		return equalHash(t.Commit, todoToFind.Hash) && t.Ref == todoToFind.Ref
 	})
 	return idx, ok
 }
@@ -147,41 +143,41 @@ func deleteTodos(todos []todo.Todo, todosToDelete []Todo) ([]todo.Todo, error) {
 	return todos, nil
 }
 
-func MoveTodosDown(fileName string, todosToMove []Todo, commentChar byte) error {
+func MoveTodosDown(fileName string, todosToMove []Todo, isInRebase bool, commentChar byte) error {
 	todos, err := ReadRebaseTodoFile(fileName, commentChar)
 	if err != nil {
 		return err
 	}
-	rearrangedTodos, err := moveTodosDown(todos, todosToMove)
+	rearrangedTodos, err := moveTodosDown(todos, todosToMove, isInRebase)
 	if err != nil {
 		return err
 	}
 	return WriteRebaseTodoFile(fileName, rearrangedTodos, commentChar)
 }
 
-func MoveTodosUp(fileName string, todosToMove []Todo, commentChar byte) error {
+func MoveTodosUp(fileName string, todosToMove []Todo, isInRebase bool, commentChar byte) error {
 	todos, err := ReadRebaseTodoFile(fileName, commentChar)
 	if err != nil {
 		return err
 	}
-	rearrangedTodos, err := moveTodosUp(todos, todosToMove)
+	rearrangedTodos, err := moveTodosUp(todos, todosToMove, isInRebase)
 	if err != nil {
 		return err
 	}
 	return WriteRebaseTodoFile(fileName, rearrangedTodos, commentChar)
 }
 
-func moveTodoDown(todos []todo.Todo, todoToMove Todo) ([]todo.Todo, error) {
-	rearrangedTodos, err := moveTodoUp(lo.Reverse(todos), todoToMove)
+func moveTodoDown(todos []todo.Todo, todoToMove Todo, isInRebase bool) ([]todo.Todo, error) {
+	rearrangedTodos, err := moveTodoUp(lo.Reverse(todos), todoToMove, isInRebase)
 	return lo.Reverse(rearrangedTodos), err
 }
 
-func moveTodosDown(todos []todo.Todo, todosToMove []Todo) ([]todo.Todo, error) {
-	rearrangedTodos, err := moveTodosUp(lo.Reverse(todos), lo.Reverse(todosToMove))
+func moveTodosDown(todos []todo.Todo, todosToMove []Todo, isInRebase bool) ([]todo.Todo, error) {
+	rearrangedTodos, err := moveTodosUp(lo.Reverse(todos), lo.Reverse(todosToMove), isInRebase)
 	return lo.Reverse(rearrangedTodos), err
 }
 
-func moveTodoUp(todos []todo.Todo, todoToMove Todo) ([]todo.Todo, error) {
+func moveTodoUp(todos []todo.Todo, todoToMove Todo, isInRebase bool) ([]todo.Todo, error) {
 	sourceIdx, ok := findTodo(todos, todoToMove)
 
 	if !ok {
@@ -194,7 +190,7 @@ func moveTodoUp(todos []todo.Todo, todoToMove Todo) ([]todo.Todo, error) {
 	// the end of the slice)
 
 	// Find the next todo that we show in lazygit's commits view (skipping the rest)
-	_, skip, ok := lo.FindIndexOf(todos[sourceIdx+1:], isRenderedTodo)
+	_, skip, ok := lo.FindIndexOf(todos[sourceIdx+1:], func(t todo.Todo) bool { return isRenderedTodo(t, isInRebase) })
 
 	if !ok {
 		// We expect callers to guard against this
@@ -208,10 +204,10 @@ func moveTodoUp(todos []todo.Todo, todoToMove Todo) ([]todo.Todo, error) {
 	return rearrangedTodos, nil
 }
 
-func moveTodosUp(todos []todo.Todo, todosToMove []Todo) ([]todo.Todo, error) {
+func moveTodosUp(todos []todo.Todo, todosToMove []Todo, isInRebase bool) ([]todo.Todo, error) {
 	for _, todoToMove := range todosToMove {
 		var newTodos []todo.Todo
-		newTodos, err := moveTodoUp(todos, todoToMove)
+		newTodos, err := moveTodoUp(todos, todoToMove, isInRebase)
 		if err != nil {
 			return nil, err
 		}
@@ -292,7 +288,33 @@ func RemoveUpdateRefsForCopiedBranch(fileName string, commentChar byte) error {
 }
 
 // We render a todo in the commits view if it's a commit or if it's an
-// update-ref. We don't render label, reset, or comment lines.
-func isRenderedTodo(t todo.Todo) bool {
-	return t.Commit != "" || t.Command == todo.UpdateRef
+// update-ref or exec. We don't render label, reset, or comment lines.
+func isRenderedTodo(t todo.Todo, isInRebase bool) bool {
+	return t.Commit != "" || (isInRebase && (t.Command == todo.UpdateRef || t.Command == todo.Exec))
+}
+
+func DropMergeCommit(fileName string, hash string, commentChar byte) error {
+	todos, err := ReadRebaseTodoFile(fileName, commentChar)
+	if err != nil {
+		return err
+	}
+
+	newTodos, err := dropMergeCommit(todos, hash)
+	if err != nil {
+		return err
+	}
+
+	return WriteRebaseTodoFile(fileName, newTodos, commentChar)
+}
+
+func dropMergeCommit(todos []todo.Todo, hash string) ([]todo.Todo, error) {
+	isMerge := func(t todo.Todo) bool {
+		return t.Command == todo.Merge && t.Flag == "-C" && equalHash(t.Commit, hash)
+	}
+	if lo.CountBy(todos, isMerge) != 1 {
+		return nil, fmt.Errorf("Expected exactly one merge commit with hash %s", hash)
+	}
+
+	_, idx, _ := lo.FindIndexOf(todos, isMerge)
+	return slices.Delete(todos, idx, idx+1), nil
 }
